"""Example settings file"""

REVEAL_META = {
    # Title of the slide
    "title": "revelation",
    # Author in the metadata of the slide
    "author": "Humberto Rocha",
    # Description in the metadata of the slide
    "description": "A revelation example presentation",
}

# Reveal markdown slide separator
REVEAL_SLIDE_SEPARATOR = "---"
REVEAL_VERTICAL_SLIDE_SEPARATOR = "---~"

# Themes
# beige, black, blood, league, moon, night, serif, simple, sky,
# solarized, white, dracula
REVEAL_THEME = "sky"

REVEAL_CONFIG = {
    "controls": False,
    "slideNumber": True,
    "hash": True,
}
